'use strict';

function prepareArrayForNgRepeat(data) {
    var array;
    if (data instanceof Array)
        array = data;
    else
        array = new Array(data);
    return array;
}

function isEmptyObj(obj) {
    if (obj instanceof Object) {
        for (var prop in obj) {
            if (obj.hasOwnProperty(prop))
                return false;
        }
        return true;
    } else
        return false;
}

function fixEmptyFields(json) {
    if (isEmptyObj(json))
        return '';
    for (var prop in json) {
        if (json.hasOwnProperty(prop)) {
            if (isEmptyObj(json[prop]))
                json[prop] = '';
            else if (json[prop] instanceof Object)
                fixEmptyFields(json[prop]);
        }
    }
    return json;
}

//SCHEMAcat angular controllers
SCHEMAcat.controller('WorkspaceCtrl', ['$scope', '$location', '$routeParams', '$timeout', 'SchemasList', 'Schema', 'UserService', 'Settings',
    function($scope, $location, $routeParams, $timeout, SchemasList, Schema, UserService, Settings) {

        function loadSchemasList(page) {
            //check if pageNumber is not a number (can happen e.g. due to manual input)
            //reload on page 1 schema index 0 if so
            if (isNaN(parseInt(page, 10)) || page < 1) {
                $location.path('workspace/page/' + 1 + '/').search({
                    'schemaIndex': 0
                });
                return;
            }

            //When the user changes page, the new AJAX data (next page) is requested by
            //the unloading controller (previous page) and used by the next loaded
            //controller (next page), thus we have to keep the server data across controller 
            //reloads by saving it in the parent scope.
            SchemasList.get({
                    userID: UserService.id,
                    startPage: page,
                    pageSize: $scope.pageSize
                },
                function(result) {
                    //we now have the pages range so we can configure the paginator and check if pageNumber is out of
                    //range (can happen e.g. due to manual input) reload on page 1 schema index 0 if so
                    if (page > Math.ceil(result['sc:result']['sc:total-schemas'] / $scope.pageSize)) {
                        $location.path('workspace/page/' + 1 + '/').search({
                            'schemaIndex': 0
                        });
                        return;
                    }

                    parentScope.totalSchemaItems = result['sc:result']['sc:total-schemas'];
                    parentScope.currentPage = Number(page);
                    //when there is just one element we create an array with it, so ng-repeat repeats just once
                    parentScope.schemasList = prepareArrayForNgRepeat(result['sc:result']['sc:schemas']['sc:schema']);
                    loadSchemaMetadata($location.search().schemaIndex);
                    parentScope.schemasResult = fixEmptyFields(result);
                });
        }


        //loads (trough a 'Schema' AJAX call) the schema metadata of the schema placed
        //in the 'index' position of the schemas list.
        function loadSchemaMetadata(index) {
            //if just changing the selected schema hide the metadata pane until the metadata is loaded
            if ($scope.animateWorkspacePageChange !== 'invisible')
                $scope.animateSchemaChange = 'invisible';
            //check if schemaIndex is out of range (can happen e.g. due to manual input)
            //reload on schema index 0 if so
            if (isNaN(parseInt(index, 10)) || index < 0 || index > $scope.schemasList.length - 1) {
                $scope.selectedSchemaIndex = undefined;
                $location.search({
                    'schemaIndex': 0
                });
                return;
            }
            //update the selected row to the route search part.
            $scope.selectedSchemaIndex = index;

            //make AJAX call to get the schema metadata for the selected row
            var selectedSchemaRow = parentScope.schemasList[Number(index)];
            Schema.get({
                    schemaID: selectedSchemaRow['xml:id']
                },
                function(result) {
                    //when there is just one element we create an array with it, so ng-repeat repeats just once
                    parentScope.filesList = prepareArrayForNgRepeat(result['sc:schema']['sc:files']['sc:file']);
                    parentScope.selectedSchema = fixEmptyFields(result);
                    parentScope.schemaEditable = UserService.id === getSchemaOwnerID();
                    parentScope.resultHasArrived = true;

                    if (Settings.getAnimationStatus()) {
                        //fade in the schema metadata
                        switch ($scope.toNextSchema) {
                            case true:
                                $scope.animateSchemaChange = 'animated fadeInUp';
                                break;
                            case false:
                                $scope.animateSchemaChange = 'animated fadeInDown';
                                break;
                                //if it is a reload toNextPage will be undefined
                            case undefined:
                                $scope.animateSchemaChange = 'animated fadeInDown';
                        }
                    } else
                        $scope.animateSchemaChange = '';
                });
        }

        function getSchemaOwnerID() {
            var schemaOwnerID;
            var authorizedUsers = prepareArrayForNgRepeat(parentScope.selectedSchema['sc:schema']['sc:access']['sc:user']);
            for (var i in authorizedUsers) {
                if (authorizedUsers[i].role === 'owner')
                    schemaOwnerID = authorizedUsers[i].ref;
            }
            return schemaOwnerID;
        }

        function updateMetadataPane() {
            $scope.workspace.namePanel.updateEditingName();
            $scope.workspace.descriptionPanel.updateEditingDescription();
            updateCollapsibleDetailPanels();
        }

        function updateCollapsibleDetailPanels() {
            var detailsPanelExpanded = false;
            for (var panel in editPanelsRoutes) {
                if ($location.search().hasOwnProperty(editPanelsRoutes[panel])) {
                    expandDetailsPanel(panel);
                    detailsPanelExpanded = true;
                }
            }
            if (!detailsPanelExpanded) {
                closeOtherCollapsibleDetailPanels('');
            }
        }

        function closeOtherCollapsibleDetailPanels(panel) {
            for (var pnl in $scope.workspace.collapsibleDetailPanels) {
                if (pnl !== panel) {
                    $scope.workspace.collapsibleDetailPanels[pnl].detailButtonTooltip = 'Show field details';
                    if ($scope.workspace.collapsibleDetailPanels[pnl].open === true) {
                        collapseDetailsPanel(pnl);
                        return true;
                    }
                }
            }
            return false;
        }

        function collapseDetailsPanel(panel) {
            if ($scope.workspace.collapsibleDetailPanels[panel].editForm &&
                $scope.workspace.collapsibleDetailPanels[panel].editForm.$dirty || $scope.workspace.collapsibleDetailPanels[panel].modelDirty)
                $scope.workspace.collapsibleDetailPanels[panel].detailButtonTooltip += ' (there are unsaved changes in this field!)';
            $scope.workspace.collapsibleDetailPanels[panel].open = false;
        }

        function expandDetailsPanel(panel) {
            if (editPanelsRoutes.hasOwnProperty(panel)) {
                if (closeOtherCollapsibleDetailPanels(panel)) {
                    //if there was other collapsible detail panel expanded, wait 350ms before expanding the clicked panel, to
                    //give time for the collpase animation of the previously open panel to finish.
                    $timeout(function() {
                        updateAndExpandPanel(panel);
                    }, 350);
                } else
                    updateAndExpandPanel(panel);
            }

            function updateAndExpandPanel(panel) {
                if ($scope.workspace.collapsibleDetailPanels[panel] !== undefined && $scope.workspace.collapsibleDetailPanels[panel].updateEditPanel !== undefined)
                    $scope.workspace.collapsibleDetailPanels[panel].updateEditPanel();
                $scope.workspace.collapsibleDetailPanels[panel].detailButtonTooltip = 'Hide field details';
                $scope.workspace.collapsibleDetailPanels[panel].open = true;
            }
        }

        //these models have to be stored in the parent scope since they are used across page changes and
        //this controller is reloaded everytime the users changes page, thus reseting the $scope models.
        var parentScope = $scope.$parent;
        parentScope.pageSize = 10;


        var editPanelsRoutes = {
            version: 'showVersionDetails',
            status: 'showStatusDetails',
            access: 'showAccessDetails',
            license: 'showLicenseDetails'
        };

        //these models are stored in the local $scope since they are not used across page changes
        $scope.selectedSchemaIndex = $routeParams.schemaIndex;

        //keep the workspace page hidden until all data is loaded            
        $scope.animateWorkspacePageChange = 'invisible';

        $scope.workspace = {};
        $scope.location = location;

        //load the schemas page and the selected schema metadata.
        //if reading the workspace for the first time i.e. not due to page change click
        if (!parentScope.workspaceFullyLoaded)
            loadSchemasList($routeParams.pageNumber);


        //update the current route when user clicks in another page in the paginator.
        //This triggers a $routeChangeStart event since we are changing the route path part.
        $scope.selectPage = function(page) {
            $location.search().schemaIndex = '0';
            delete $location.search().show;
            $location.path('workspace.html/page/' + page + '/').search($location.search());
        };

        //update the current route when the uses clicks one schema in the list. This triggers
        //a $routeUpdate event since we are just changing the route search part.
        $scope.selectSchema = function(rowIndex) {
            $location.search().schemaIndex = rowIndex.toString();
            delete $location.search().show;
            $location.search($location.search());
        };

        $scope.toggleDetailsPanel = function(panel) {
            var route = editPanelsRoutes[panel];
            //check if we are collapsing or expanding the panel
            if ($location.search()[route]) {
                //collapsing -> delete panel route
                delete $location.search()[route];
            } else {
                //expanding -> 1. clean routes from other detail panels if they are open
                for (var prop in editPanelsRoutes) {
                    if (editPanelsRoutes[prop] !== route)
                        delete $location.search()[editPanelsRoutes[prop]];
                }
                //2. update the route to display the clicked panel
                $location.search()[editPanelsRoutes[panel]] = true;
            }
            //apply the new route
            $location.search($location.search());
        };

        //listen for the $routeChangeStart event. Triggered when route path is changed (page change).
        $scope.$on('$routeChangeStart', function(event, next, current) {
            parentScope.resultHasArrived = false;
            if (Settings.getAnimationStatus()) {
                if (next.params !== undefined &&
                    next.params.pageNumber !== undefined &&
                    next.params.pageNumber > current.params.pageNumber)
                    parentScope.toNextPage = true;
                else
                    parentScope.toNextPage = false;

                //slide out workspace page
                switch (next.templateUrl) {
                    case 'partials/workspace.html':
                        $scope.animateWorkspacePageChange =
                            parentScope.toNextPage ?
                            'animated slideOutLeft' : 'animated slideOutRight';
                        break;
                    case 'partials/contact.html':
                    case 'partials/about.html':
                        $scope.animateWorkspacePageChange = 'animated slideOutLeft';
                        break;
                    default:
                        $scope.animateWorkspacePageChange = 'animated fadeOutDown';
                }
            }
            if (next.templateUrl === 'partials/workspace.html')
                loadSchemasList(next.params.pageNumber);
            else
                parentScope.workspaceFullyLoaded = false;
        });

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            //when coming back to the workspace from other routes, the workspace should apear from the left to the right
            if (Settings.getAnimationStatus()) {
                if (previous === undefined || previous.templateUrl === undefined ||
                    previous.templateUrl === 'partials/accountDetails.html')
                    parentScope.toNextPage = undefined;
            }
        });

        //listen for the $routeUpdate event and update the selected schema metadata.
        //(if not done, the selection highlight is not updated e.g. when
        //navigating back and forth)
        $scope.$on('$routeUpdate', function(event, route) {
            //if selected index changed, load the new schema.
            if (route.templateUrl === 'partials/workspace.html' && route.params.schemaIndex !== $scope.selectedSchemaIndex) {
                parentScope.resultHasArrived = false;
                if (Settings.getAnimationStatus()) {
                    $scope.toNextSchema =
                        route.params.schemaIndex > $scope.selectedSchemaIndex ?
                        true : false;
                    $scope.animateSchemaChange =
                        $scope.toNextSchema ?
                        'animated fadeOutUp' : 'animated fadeOutDown';
                    $timeout(function() {
                        loadSchemaMetadata(route.params.schemaIndex);
                    }, 250);
                } else
                    loadSchemaMetadata(route.params.schemaIndex);

                //register a listener to update the metadata pane once the result has arrived
                var watchListener = $scope.$watch('resultHasArrived', function(newValue, oldValue) {
                    if (newValue) {
                        updateMetadataPane();
                        //derregister the listener since the workspace page is now loaded
                        watchListener();
                    }
                });
            } else
                updateMetadataPane();
        });


        //watch for our AJAX data to arrive and fade in workspace afterwards.
        //when changing page, the data is requested before the workspace controller is reloaded
        //so we should fade in the workspace in the new controller instance after reload.
        var watchListener = $scope.$watch('resultHasArrived', function(newValue, oldValue) {
            if (newValue) {
                if (Settings.getAnimationStatus()) {
                    //fade in the workspace page
                    switch (parentScope.toNextPage) {
                        case false:
                            $scope.animateWorkspacePageChange = 'animated slideInLeft';
                            break;
                        case true:
                            $scope.animateWorkspacePageChange = 'animated slideInRight';
                            break;
                            //if it is a reload toNextPage will be undefined
                        default:
                            $scope.animateWorkspacePageChange = 'animated fadeInDown';
                    }

                    $timeout(function() {
                        $scope.animateWorkspacePageChange = '';
                    }, 250);
                } else
                    $scope.animateWorkspacePageChange = '';
                updateMetadataPane();
                parentScope.workspaceFullyLoaded = true;
                //derregister the listener since the workspace page is now loaded
                watchListener();
            }
        });
    }
]);

SCHEMAcat.controller('NamePanelCtrl', ['$scope', '$location', '$timeout', 'Schema',
    function($scope, $location, $timeout, Schema) {

        $scope.workspace.namePanel = {};
        var originalNameValue;

        $scope.workspace.namePanel.updateEditingName = function() {
            if ($location.search().editingName) {
                if ($scope.schemaEditable) {
                    $scope.workspace.namePanel.editButtonTooltip = 'Hide edit name';
                    if (originalNameValue === undefined)
                        originalNameValue = $scope.selectedSchema['sc:schema']['sc:name'];
                    $scope.workspace.namePanel.editForm.open = true;
                } else {
                    delete $location.search().editingName;
                    $scope.workspace.namePanel.editButtonTooltip = 'Edit schema name';
                    if ($scope.workspace.namePanel.editForm.input.$dirty)
                        $scope.workspace.namePanel.editButtonTooltip += ' (there are unsaved changes in this field!)';
                    $location.search($location.search());
                }
            } else {
                $scope.workspace.namePanel.editForm.open = false;
                $scope.workspace.namePanel.editButtonTooltip = 'Edit schema name';
                if ($scope.workspace.namePanel.editForm.input.$dirty)
                    $scope.workspace.namePanel.editButtonTooltip += ' (there are unsaved changes in this field!)';
            }
            $scope.workspace.namePanel.updateAjaxError = false;
        };

        $scope.saveName = function() {
            var name = $scope.selectedSchema['sc:schema']['sc:name'];
            var schema = new Schema();
            $scope.workspace.namePanel.waitingForServerUpdate = true;
            $scope.workspace.namePanel.updateAjaxError = false;

            schema.name = name;

            schema.$save({
                    schemaID: $scope.schemasList[$scope.selectedSchemaIndex]['xml:id'],
                    schemaField: 'name'
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success) {
                        $scope.selectedSchema['sc:schema']['sc:name'] = fixEmptyFields(result['sc:response']['sc:written-data']['sc:name']);
                        originalNameValue = result['sc:response']['sc:written-data']['sc:name'];
                        $scope.workspace.namePanel.waitingForServerUpdate = false;
                        $scope.workspace.namePanel.editForm.$setPristine();
                        delete $location.search().editingName;
                        $location.search($location.search());
                    } else {
                        //server reports an handeled error
                        $scope.workspace.namePanel.waitingForServerUpdate = false;
                        $scope.workspace.namePanel.updateAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.workspace.namePanel.waitingForServerUpdate = false;
                    $scope.workspace.namePanel.updateAjaxError = true;
                });
        };

        $scope.toggleEditName = function(cancel) {
            if ($location.search().editingName) {
                delete $location.search().editingName;
                if (cancel) {
                    //this timeout is so that the name does collapse abruptly
                    $timeout(function() {
                        $scope.selectedSchema['sc:schema']['sc:name'] = originalNameValue;
                        $scope.workspace.namePanel.editForm.$setPristine();
                        $scope.$apply();
                        $location.search($location.search());
                    }, 150);
                    return;
                }
            } else {
                $location.search().editingName = true;
            }
            $location.search($location.search());
        };

    }
]);

SCHEMAcat.controller('DescriptionPanelCtrl', ['$scope', '$location', '$timeout', 'Schema',
    function($scope, $location, $timeout, Schema) {

        $scope.workspace.descriptionPanel = {};
        var originalDescriptionValue;

        $scope.workspace.descriptionPanel.updateEditingDescription = function() {
            if ($location.search().editingDescription) {
                if ($scope.schemaEditable) {
                    //If the textarea was collapsed when its model was first changed, autosizeTextarea does not set 
                    //the textarea height by design. Thus we resize the textarea when it is expanded.
                    //When reloading or loading the workspace for the first time, the resize method from autosizeTextarea
                    //is already trigered (for non-collapsed textareas) by the change in ngModel. So do not broadcast a autosizeTextarea.resize event.
                    if ($scope.workspaceFullyLoaded)
                        $scope.$broadcast('autosizeTextarea.resize');
                    $scope.workspace.descriptionPanel.editButtonTooltip = 'Hide edit description';
                    if (originalDescriptionValue === undefined)
                        originalDescriptionValue = $scope.selectedSchema['sc:schema']['sc:description'];
                    $scope.workspace.descriptionPanel.editForm.open = true;
                } else {
                    delete $location.search().editingDescription;
                    $scope.workspace.descriptionPanel.editButtonTooltip = 'Edit description';
                    if ($scope.workspace.descriptionPanel.editForm.input.$dirty)
                        $scope.workspace.descriptionPanel.editButtonTooltip += ' (there are unsaved changes in this field!)';
                    $location.search($location.search());
                }
            } else {
                $scope.workspace.descriptionPanel.editForm.open = false;
                $scope.workspace.descriptionPanel.editButtonTooltip = 'Edit description';
                if ($scope.workspace.descriptionPanel.editForm.input.$dirty)
                    $scope.workspace.descriptionPanel.editButtonTooltip += ' (there are unsaved changes in this field!)';
            }
            $scope.workspace.descriptionPanel.updateAjaxError = false;
        };

        $scope.saveDescription = function() {
            var description = $scope.selectedSchema['sc:schema']['sc:description'];
            var schema = new Schema();
            $scope.workspace.descriptionPanel.waitingForServerUpdate = true;
            $scope.workspace.descriptionPanel.updateAjaxError = false;

            schema.description = description;

            schema.$save({
                    schemaID: $scope.schemasList[$scope.selectedSchemaIndex]['xml:id'],
                    schemaField: 'description'
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success) {
                        $scope.selectedSchema['sc:schema']['sc:description'] = fixEmptyFields(result['sc:response']['sc:written-data']['sc:description']);
                        originalDescriptionValue = result['sc:response']['sc:written-data']['sc:description'];
                        $scope.workspace.descriptionPanel.waitingForServerUpdate = false;
                        $scope.workspace.descriptionPanel.editForm.$setPristine();
                        delete $location.search().editingDescription;
                        $location.search($location.search());
                    } else {
                        //server reports an handeled error
                        $scope.workspace.descriptionPanel.waitingForServerUpdate = false;
                        $scope.workspace.descriptionPanel.updateAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.workspace.descriptionPanel.waitingForServerUpdate = false;
                    $scope.workspace.descriptionPanel.updateAjaxError = true;
                });
        };

        $scope.toggleEditDescription = function(cancel) {
            if ($location.search().editingDescription) {
                delete $location.search().editingDescription;
                if (cancel) {
                    //this timeout is so that the description does collapse abruptly
                    $timeout(function() {
                        $scope.selectedSchema['sc:schema']['sc:description'] = originalDescriptionValue;
                        $scope.workspace.descriptionPanel.editForm.$setPristine();
                        $scope.$apply();
                        $location.search($location.search());
                    }, 150);
                    return;
                }
            } else {
                $location.search().editingDescription = true;
            }
            $location.search($location.search());
        };
    }
]);

SCHEMAcat.controller('VersionDetailsPanelCtrl', ['$scope', '$location', '$timeout', 'Schema',
    function($scope, $location, $timeout, Schema) {

        $scope.workspace.collapsibleDetailPanels.version = {};
        $scope.workspace.collapsibleDetailPanels.version.editPanel = {};
        $scope.workspace.collapsibleDetailPanels.version.editPanel.open = false;
        var originalVersionValue;

        $scope.toggleEditVersion = function(cancel) {
            if ($location.search().editingVersion) {
                delete $location.search().editingVersion;
                if (cancel) {
                    //this timeout is so that the description does collapse abruptly
                    $timeout(function() {
                        $scope.workspace.collapsibleDetailPanels.version.editForm.$setPristine();
                        $scope.$apply();
                        $location.search($location.search());
                        //after collapsing the edit panel collapse the fields displaying the old value and reexpand them after 350ms
                        $scope.workspace.collapsibleDetailPanels.version.resettingUpdate = true;
                        $timeout(function() {
                            $scope.selectedSchema['sc:schema']['sc:version'] = originalVersionValue;
                            //expand the value fiels with the original value
                            $scope.workspace.collapsibleDetailPanels.version.resettingUpdate = false;
                        }, 350);
                    }, 150);
                    return;
                }
            } else
                $location.search().editingVersion = true;

            $location.search($location.search());
        };

        $scope.workspace.collapsibleDetailPanels.version.updateEditPanel = function() {
            if ($location.search().editingVersion) {
                if ($scope.schemaEditable) {
                    $scope.workspace.collapsibleDetailPanels.version.editButtonTooltip = 'Hide edit panel';
                    if (originalVersionValue === undefined)
                        originalVersionValue = $scope.selectedSchema['sc:schema']['sc:version'];
                    $scope.workspace.collapsibleDetailPanels.version.editPanel.open = true;
                } else {
                    delete $location.search().editingVersion;
                    $scope.workspace.collapsibleDetailPanels.version.editButtonTooltip = 'Change schema version';
                    if ($scope.workspace.collapsibleDetailPanels.version.editForm && $scope.workspace.collapsibleDetailPanels.version.editForm.input.$dirty)
                        $scope.workspace.collapsibleDetailPanels.version.editButtonTooltip += ' (there are unsaved changes in this field!)';
                    $location.search($location.search());
                }
            } else {
                $scope.workspace.collapsibleDetailPanels.version.editPanel.open = false;
                $scope.workspace.collapsibleDetailPanels.version.editButtonTooltip = 'Change schema version';
                if ($scope.workspace.collapsibleDetailPanels.version.editForm && $scope.workspace.collapsibleDetailPanels.version.editForm.input.$dirty)
                    $scope.workspace.collapsibleDetailPanels.version.editButtonTooltip += ' (there are unsaved changes in this field!)';
            }
            $scope.workspace.collapsibleDetailPanels.version.updateAjaxError = false;
        };

        $scope.saveVersion = function() {
            var version = $scope.selectedSchema['sc:schema']['sc:version'];
            var schema = new Schema();
            $scope.workspace.collapsibleDetailPanels.version.waitingForServerUpdate = true;
            $scope.workspace.collapsibleDetailPanels.version.updateAjaxError = false;

            schema.version = version;

            schema.$save({
                    schemaID: $scope.selectedSchema['sc:schema']['xml:id'],
                    schemaField: 'version'
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success) {
                        $scope.selectedSchema['sc:schema']['sc:version'] = fixEmptyFields(result['sc:response']['sc:written-data']['sc:version']);
                        originalVersionValue = $scope.selectedSchema['sc:schema']['sc:version'];
                        $scope.schemasList[$scope.selectedSchemaIndex]['sc:version'] = originalVersionValue;
                        $scope.workspace.collapsibleDetailPanels.version.waitingForServerUpdate = false;
                        $scope.workspace.collapsibleDetailPanels.version.editForm.$setPristine();
                        delete $location.search().editingVersion;
                        $location.search($location.search());
                    } else {
                        //server reports an handeled error
                        $scope.workspace.collapsibleDetailPanels.version.waitingForServerUpdate = false;
                        $scope.workspace.collapsibleDetailPanels.version.updateAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.workspace.collapsibleDetailPanels.version.waitingForServerUpdate = false;
                    $scope.workspace.collapsibleDetailPanels.version.updateAjaxError = true;
                });
        };
    }
]);

SCHEMAcat.controller('StatusDetailsPanelCtrl', ['$scope', '$location', 'Schema',
    function($scope, $location, Schema) {
        $scope.workspace.collapsibleDetailPanels.status = {};
        $scope.workspace.collapsibleDetailPanels.status.editPanel = {};
        $scope.workspace.collapsibleDetailPanels.status.editPanel.open = false;
        $scope.workspace.collapsibleDetailPanels.status.possibleStatusList = [];

        var privateAccessPossibleStatusList = [
            'development'
        ];

        var publicAccessPossibleStatusList = [
            'production'
        ];

        var commonPossibleStatusList = [
            'deprecated',
            'superseeded'
        ];

        $scope.toggleEditStatus = function() {
            if ($location.search().editingStatus)
                delete $location.search().editingStatus;
            else
                $location.search().editingStatus = true;

            $location.search($location.search());
        };

        $scope.workspace.collapsibleDetailPanels.status.updateEditPanel = function() {
            if ($location.search().editingStatus) {
                if ($scope.schemaEditable) {
                    if ($scope.selectedSchema['sc:schema']['sc:access']['sc:all'] && $scope.selectedSchema['sc:schema']['sc:access']['sc:all'].actions === 'r')
                        $scope.workspace.collapsibleDetailPanels.status.possibleStatusList = publicAccessPossibleStatusList.concat(commonPossibleStatusList);
                    else
                        $scope.workspace.collapsibleDetailPanels.status.possibleStatusList = privateAccessPossibleStatusList.concat(commonPossibleStatusList);

                    $scope.workspace.collapsibleDetailPanels.status.editPanel.open = true;
                } else {
                    delete $location.search().editingStatus;
                    $location.search($location.search());
                }
            } else
                $scope.workspace.collapsibleDetailPanels.status.editPanel.open = false;
            $scope.workspace.collapsibleDetailPanels.status.updateAjaxError = false;
        };

        $scope.saveStatus = function() {
            var status = $scope.selectedSchema['sc:schema']['sc:status'];
            var schema = new Schema();
            $scope.workspace.collapsibleDetailPanels.status.waitingForServerUpdate = true;
            $scope.workspace.collapsibleDetailPanels.status.updateAjaxError = false;

            schema.status = status;

            schema.$save({
                    schemaID: $scope.selectedSchema['sc:schema']['xml:id'],
                    schemaField: 'status'
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success) {
                        $scope.selectedSchema['sc:schema']['sc:status'] = fixEmptyFields(result['sc:response']['sc:written-data']['sc:status']);
                        $scope.schemasList[$scope.selectedSchemaIndex]['sc:status'] = $scope.selectedSchema['sc:schema']['sc:status'];
                        $scope.workspace.collapsibleDetailPanels.status.editForm.$setPristine();
                        $scope.workspace.collapsibleDetailPanels.status.waitingForServerUpdate = false;
                    } else {
                        //server reports an handeled error
                        $scope.workspace.collapsibleDetailPanels.status.waitingForServerUpdate = false;
                        $scope.workspace.collapsibleDetailPanels.status.updateAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.workspace.collapsibleDetailPanels.status.waitingForServerUpdate = false;
                    $scope.workspace.collapsibleDetailPanels.status.updateAjaxError = true;
                });
        };
    }
]);

SCHEMAcat.controller('AccessDetailsPanelCtrl', ['$scope', '$location', 'Schema',
    function($scope, $location, Schema) {

        $scope.workspace.collapsibleDetailPanels.access = {};
        $scope.workspace.collapsibleDetailPanels.access.editPanel = {};
        $scope.workspace.collapsibleDetailPanels.access.editPanel.open = false;
        $scope.currentAccessPolicy = '';

        $scope.toggleEditAccess = function() {
            if ($location.search().editingAccess)
                delete $location.search().editingAccess;
            else
                $location.search().editingAccess = true;

            $location.search($location.search());
        };

        $scope.workspace.collapsibleDetailPanels.access.updateEditPanel = function() {
            if ($location.search().editingAccess) {
                if ($scope.schemaEditable) {
                    $scope.currentAccessPolicy = $scope.workspace.collapsibleDetailPanels.access.getAccessPolicy();
                    $scope.workspace.collapsibleDetailPanels.access.editPanel.open = true;
                } else {
                    delete $location.search().editingAccess;
                    $location.search($location.search());
                }
            } else
                $scope.workspace.collapsibleDetailPanels.access.editPanel.open = false;
            $scope.workspace.collapsibleDetailPanels.access.updateAjaxError = false;
        };

        $scope.workspace.collapsibleDetailPanels.access.getAccessPolicy = function() {
            if ($scope.selectedSchema['sc:schema']['sc:access']['sc:all'] !== undefined && $scope.selectedSchema['sc:schema']['sc:access']['sc:all'].actions === 'r')
                return 'shareURL';
            else
                return 'private';
        };

        $scope.saveAccessStatus = function(newAccessStatus) {
            var accessUser = $scope.selectedSchema['sc:schema']['sc:access']['sc:user'];
            var schema = new Schema();
            $scope.workspace.collapsibleDetailPanels.access.waitingForServerUpdate = true;
            $scope.workspace.collapsibleDetailPanels.access.updateAjaxError = false;

            switch (newAccessStatus) {
                case 'shareURL':
                    schema.access = {
                        'all': {
                            'actions': 'r'
                        },
                        'user': accessUser
                    };
                    break;
                case 'private':
                    schema.access = {
                        'user': accessUser
                    };
                    break;
            }

            schema.$save({
                    schemaID: $scope.selectedSchema['sc:schema']['xml:id'],
                    schemaField: 'access'
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success) {
                        $scope.selectedSchema['sc:schema']['sc:access'] = result['sc:response']['sc:written-data']['sc:access'];
                        $scope.workspace.collapsibleDetailPanels.access.editForm.$setPristine();
                        $scope.workspace.collapsibleDetailPanels.access.waitingForServerUpdate = false;
                    } else {
                        //server reports an handeled error
                        $scope.workspace.collapsibleDetailPanels.access.waitingForServerUpdate = false;
                        $scope.workspace.collapsibleDetailPanels.access.updateAjaxError = true;
                    }
                    //refresh the view model
                    $scope.currentAccessPolicy = $scope.workspace.collapsibleDetailPanels.access.getAccessPolicy();
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.workspace.collapsibleDetailPanels.access.waitingForServerUpdate = false;
                    $scope.workspace.collapsibleDetailPanels.access.updateAjaxError = true;
                    $scope.currentAccessPolicy = $scope.workspace.collapsibleDetailPanels.access.getAccessPolicy();
                });
        };
    }
]);



SCHEMAcat.controller('LicenseDetailsPanelCtrl', ['$scope', '$modal', '$location', '$timeout',
    'Schema', 'Licenses',
    function($scope, $modal, $location, $timeout, Schema, Licenses) {

        $scope.workspace.collapsibleDetailPanels.license = {};
        $scope.input = {};

        var originalLicenseValue;
        var modalInstance;


        $scope.filterLicenses = function(item) {
            var regex = new RegExp($scope.workspace.collapsibleDetailPanels.license.editForm.filterTerm.$modelValue, 'i');
            if (item['sc:full-name'].match(regex) || item['xml:id'].match(regex) || $scope.workspace.collapsibleDetailPanels.license.editForm.filterTerm.$modelValue === undefined) {
                return true;
            }
            return false;
        };

        $scope.showSelectedLicense = function(license) {
            if ($scope.selectedSchema['sc:schema']['sc:license']['xml:id'] !== license['xml:id']) {
                $scope.workspace.collapsibleDetailPanels.license.modelDirty = true;
                $scope.workspace.collapsibleDetailPanels.license.updatingLicense = true;
                $timeout(function() {
                    $scope.selectedSchema['sc:schema']['sc:license'] = license;
                    $scope.workspace.collapsibleDetailPanels.license.updatingLicense = false;
                }, 250);
            }
        };

        $scope.saveNewLicense = function() {
            var license = $scope.selectedSchema['sc:schema']['sc:license'];
            var schema = new Schema();
            $scope.workspace.collapsibleDetailPanels.license.waitingForServerUpdate = true;
            $scope.workspace.collapsibleDetailPanels.license.updateAjaxError = false;

            schema.license = license['xml:id'];
            schema.$save({
                    schemaID: $scope.selectedSchema['sc:schema']['xml:id'],
                    schemaField: 'license'
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success &&
                        result['sc:response']['sc:written-data']['sc:license'].ref === $scope.selectedSchema['sc:schema']['sc:license']['xml:id']) {

                        originalLicenseValue = $scope.selectedSchema['sc:schema']['sc:license'];
                        $scope.workspace.collapsibleDetailPanels.license.modelDirty = false;
                        $scope.workspace.collapsibleDetailPanels.license.editForm.$setPristine();
                        $scope.workspace.collapsibleDetailPanels.license.waitingForServerUpdate = false;
                        $scope.triggerCloseModalRoute();
                    } else {
                        //server reports an handeled error
                        $scope.workspace.collapsibleDetailPanels.license.waitingForServerUpdate = false;
                        $scope.workspace.collapsibleDetailPanels.license.updateAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.selectedSchema['sc:schema']['sc:license'] = originalLicenseValue;
                    $scope.workspace.collapsibleDetailPanels.license.waitingForServerUpdate = false;
                    $scope.workspace.collapsibleDetailPanels.license.updateAjaxError = true;
                });
        };

        $scope.triggerOpenModalRoute = function() {
            if (!$location.search().editingLicense) {
                $location.search().editingLicense = true;
                $location.search($location.search());
            }
        };

        $scope.triggerCloseModalRoute = function() {
            if ($location.search().editingLicense) {
                delete $location.search().editingLicense;
                $location.search($location.search());
            }
        };

        //this function evaluates changes in the route to open and close the modal
        $scope.workspace.collapsibleDetailPanels.license.updateEditPanel = function() {
            if ($location.search().editingLicense) {
                if ($scope.schemaEditable) {
                    if (originalLicenseValue === undefined)
                        originalLicenseValue = $scope.selectedSchema['sc:schema']['sc:license'];

                    if ($scope.workspace.collapsibleDetailPanels.license.licenseList === undefined) {
                        $scope.workspace.collapsibleDetailPanels.license.licensesArrived = false;
                        //get the list of licenses
                        Licenses.get(
                            function(result) {
                                $scope.workspace.collapsibleDetailPanels.license.licenseList = result;
                                for (var i = 0; i < result['sc:licenses']['sc:license'].length; i++) {
                                    if (result['sc:licenses']['sc:license'][i]['xml:id'] === $scope.selectedSchema['sc:schema']['sc:license']['xml:id'])
                                        $scope.input.model = $scope.workspace.collapsibleDetailPanels.license.licenseList['sc:licenses']['sc:license'][i]['xml:id'];
                                }
                                $scope.workspace.collapsibleDetailPanels.license.licensesArrived = true;

                            });
                    }
                    openModal();
                } else {
                    $scope.workspace.collapsibleDetailPanels.license.editButtonTooltip = 'Change schema license';
                    if ($scope.workspace.collapsibleDetailPanels.license.editForm && $scope.workspace.collapsibleDetailPanels.license.modelDirty)
                        $scope.workspace.collapsibleDetailPanels.license.editButtonTooltip += ' (there are unsaved changes in this field!)';
                    $scope.triggerCloseModalRoute();
                }
            } else if (modalInstance !== undefined) {
                modalInstance.dismiss('dismiss');
            }
            $scope.workspace.collapsibleDetailPanels.license.editButtonTooltip = 'Change schema license';
            if ($scope.workspace.collapsibleDetailPanels.license.editForm && $scope.workspace.collapsibleDetailPanels.license.modelDirty)
                $scope.workspace.collapsibleDetailPanels.license.editButtonTooltip += ' (there are unsaved changes in this field!)';
        };

        function openModal() {
            modalInstance = $modal.open({
                scope: $scope,
                templateUrl: 'partials/editLicenseModal.html',
            });

            modalInstance.result.then(function() {
                //function runs if modal is closed with a result ($modalInstance.close(result)) i.e. click 'save'
                modalInstance = undefined;
            }, function(reason) {
                //function runs if modal is dismissed i.e. click 'cancel', 'x', backdrop or ESC key press
                modalInstance = undefined;
                //if backdrop click or ESC key, update route.
                if (reason !== 'dismiss')
                    $scope.triggerCloseModalRoute();
            });
        }

        $scope.cancelEdit = function() {
            if ($scope.workspace.collapsibleDetailPanels.license.modelDirty) {
                $scope.workspace.collapsibleDetailPanels.license.updatingLicense = true;
                $scope.workspace.collapsibleDetailPanels.license.modelDirty = false;
                $timeout(function() {
                    $scope.selectedSchema['sc:schema']['sc:license'] = originalLicenseValue;
                    $scope.workspace.collapsibleDetailPanels.license.updatingLicense = false;
                }, 250);
                $scope.workspace.collapsibleDetailPanels.license.editForm.$setPristine();
            }
            $scope.triggerCloseModalRoute();
        };
    }
]);


SCHEMAcat.controller('FileController', ['$scope', '$http', '$timeout', '$location',
    function($scope, $http, $timeout, $location) {

        updateShowFiles();

        //show-hide file toggle handler
        $scope.showFile = function(fileName) {
            //update the search part of the route
            //triggers '$routeUpdate'
            $scope.showFileAjaxError = false;
            $timeout(function() { //(wrapping in $timeout is neded for Firefox)
                var locationSearch = $location.search();
                if ($scope.showSchemaBtn === true) {
                    if (locationSearch.show === undefined)
                        locationSearch.show = '#' + fileName;
                    else if (locationSearch.show.indexOf(fileName) === -1)
                        locationSearch.show = locationSearch.show + '#' + fileName;
                } else {
                    for (var prop in locationSearch) {
                        if (prop === 'show') {
                            var newLocationSearchShowArray = locationSearch[prop].split('#');
                            for (var i in newLocationSearchShowArray) {
                                if (newLocationSearchShowArray[i] === fileName)
                                    newLocationSearchShowArray.splice(i, 1);
                            }

                            var newLocationSearchShowString = newLocationSearchShowArray.join('#');
                            if (newLocationSearchShowString !== '') {
                                locationSearch[prop] = newLocationSearchShowString;
                            } else
                                delete locationSearch[prop];
                        }
                    }
                }
                $location.search(locationSearch);
            });
        };

        $scope.$on('$routeUpdate', function(event, route) {
            if ($scope.resultHasArrived)
                updateShowFiles();
        });

        function updateShowFiles() {
            if ($location.search().show !== undefined && $location.search().show.indexOf($scope.file['sc:name']) !== -1) {
                $scope.showSchemaBtn = true;
                displayFile($scope.selectedSchema['sc:schema']['xml:id'], $scope.file['sc:name']);
            } else {
                $scope.showSchemaBtn = false;
                $scope.file.open = false;
            }
        }

        function displayFile(schemaID, fileName) {
            if ($scope.schemaFileContent === undefined) {
                $http.get('/schemacat/schemas/' + schemaID + '/files/' + fileName).
                success(function(data, status, headers, config) {
                    $scope.prettify = 'prettyprint linenums';

                    //execute prettyPrint asynchronously. We use $timeout since prettyPrint() 
                    //must run after DOM rendering
                    $timeout(function() {
                        prettyPrint();
                        $scope.file.open = true;
                    });

                    $scope.schemaFileContent = data;
                }).
                error(function(data, status, headers, config) {
                    $scope.showSchemaBtn = false;
                    $scope.showFile(fileName);
                    $scope.showFileAjaxError = true;
                });
            } else
                $scope.file.open = true;
        }

    }
]);


SCHEMAcat.controller('TopMenuCtrl', ['$scope', '$modal', '$location', 'UserService', 'Settings', 'ShibService',
    function($scope, $modal, $location, UserService, Settings, ShibService) {

        $scope.displayName = UserService.displayName;
        $scope.shibURL = ShibService.url;

        //watch for changes in the displayName since it can be changed by the user in the account details page
        $scope.$watch(
            function() {
                return UserService.displayName;
            },
            function(newValue, oldValue) {
                if (newValue !== oldValue) {
                    $scope.displayName = newValue;
                }
            });

        updateTopMenu();

        $scope.isActive = function(viewLocation) {
            if (viewLocation === '/workspace.html' && $location.path().indexOf('/workspace.html/') !== -1)
                return true;
            return viewLocation === $location.path();
        };

        $scope.updateAnimationStatus = function() {
            if ($location.search().animationsDisabled)
                delete $location.search().animationsDisabled;
            else
                $location.search().animationsDisabled = true;
            $location.search($location.search());
        };

        $scope.toggleTopMenu = function() {
            if ($location.search().topMenuExpanded)
                delete $location.search().topMenuExpanded;
            else
                $location.search().topMenuExpanded = true;

            $location.search($location.search());
        };

        $scope.animationsUrlPart = function() {
            if ($location.search().animationsDisabled)
                return 'animationsDisabled';
            else
                return '';
        };

        $scope.topMenuExpandedUrlPart = function() {
            if ($location.search().topMenuExpanded)
                return 'topMenuExpanded';
            else
                return '';
        };

        $scope.$on('$routeUpdate', function(event, route) {
            updateTopMenu();
        });

        //Modal window handlers and options
        $scope.loginClick = function() {
            var modalInstance = $modal.open({
                scope: $scope,
                templateUrl: 'partials/loginModal.html',
                controller: 'LoginModalCtrl'
            });

            modalInstance.result.then(function(loginName) {
                alert('Now set the user cookie to [' + loginName + ']');
                document.cookie = 'USER=' + loginName + ';domain=;path=/';
                document.location.hash = '';
                document.location.reload(true);
            });
        };

        function updateTopMenu() {
            $scope.topMenuCollapsed = !$location.search().topMenuExpanded;

            Settings.setAnimationStatus(!$location.search().animationsDisabled);
            $scope.animationsChecked = Settings.getAnimationStatus();
        }

    }
]);

SCHEMAcat.controller('LoginModalCtrl', ['$scope', '$modalInstance',
    function($scope, $modalInstance) {
        $scope.loginClose = function() {
            $modalInstance.dismiss('cancel');
        };

        //set cookie code. REMOVE in production
        $scope.loginSubmit = function() {
            $modalInstance.close(this.loginNameInput.text);
        };
    }
]);

SCHEMAcat.controller('AboutPageCtrl', ['$scope', 'Settings',
    function($scope, Settings) {

        //When the user loads (or reloads) the 'About' page as the first page to be accessed, the 'toNextPage' variable
        //used by the workspace to determine the direction of its enter animation is: 'undefined'. Thus we set it so
        //that if the user moves to the workspace ('Home'), the workspace will enter with the right animation.
        //This is done independently of 'Settings.getAnimationStatus()' so when the animations are turned on, their
        //governing variables are already set.
        $scope.$parent.toNextPage = false;

        if (Settings.getAnimationStatus()) {
            $scope.animateInOut = 'animated slideInRight';
        }

        $scope.$on('$routeChangeStart', function(event, next, current) {
            if (Settings.getAnimationStatus()) {
                switch (next.templateUrl) {
                    case 'partials/workspace.html':
                        $scope.animateInOut = 'animated slideOutRight';
                        break;
                    case 'partials/contact.html':
                        $scope.animateInOut = 'animated slideOutLeft';
                        break;
                    default:
                        $scope.animateInOut = 'animated fadeOutDown';
                }
            }
        });

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            if (Settings.getAnimationStatus()) {
                var previousUrl = previous !== undefined ?
                    previous.templateUrl : undefined;
                switch (previousUrl) {
                    case 'partials/workspace.html':
                        $scope.animateInOut = 'animated slideInRight';
                        break;
                    case 'partials/contact.html':
                        $scope.animateInOut = 'animated slideInLeft';
                        break;
                    default:
                        $scope.animateInOut = 'animated fadeInDown';
                }
            }
        });
    }
]);

SCHEMAcat.controller('ContactPageCtrl', ['$scope', 'Settings',
    function($scope, Settings) {

        //When the user loads (or reloads) the 'Contact' page as the first page to be accessed, the 'toNextPage' variable
        //used by the workspace to determine the direction of its enter animation is: 'undefined'. Thus we set it so
        //that if the user moves to the workspace ('Home'), the workspace will enter with the right animation.
        //This is done independently of 'Settings.getAnimationStatus()' so when the animations are turned on, their
        //governing variables are already set.
        $scope.$parent.toNextPage = false;

        if (Settings.getAnimationStatus()) {
            $scope.animateInOut = 'animated slideInRight';
        }

        $scope.$on('$routeChangeStart', function(event, next, current) {
            if (Settings.getAnimationStatus()) {
                switch (next.templateUrl) {
                    case 'partials/about.html':
                    case 'partials/contact.html':
                    case 'partials/workspace.html':
                        $scope.animateInOut = 'animated slideOutRight';
                        break;
                    default:
                        $scope.animateInOut = 'animated fadeOutDown';
                }
            }
        });

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            //when coming back to the workspace from other routes, the workspace should apear from the left to the right
            if (Settings.getAnimationStatus()) {
                var previousUrl = previous !== undefined ?
                    previous.templateUrl : undefined;
                switch (previousUrl) {
                    case 'partials/about.html':
                    case 'partials/contact.html':
                    case 'partials/workspace.html':
                        $scope.animateInOut = 'animated slideInRight';
                        break;
                    default:
                        $scope.animateInOut = 'animated fadeInDown';
                }
            }
        });
    }
]);

SCHEMAcat.controller('AccountDetailsPageCtrl', ['$scope', '$location', 'User', 'UserService', 'Settings',
    function($scope, $location, User, UserService, Settings) {

        //this function is used to clean outdated emails from the route search part
        //i.e. after editing an email the user can still go back in the browser but
        //the old email addess does not exist anymore, thus we clean it when back is clicked.
        function cleanOutdatedSearchPathEmails() {
            var locationSearch = $location.search();
            for (var prop in locationSearch) {
                if (prop === 'editingEmail') {
                    var newLocationSearcheditingEmailArray = locationSearch[prop].split('#');
                    for (var i = 0; i < newLocationSearcheditingEmailArray.length; i++) {
                        var found = false;
                        for (var j = 0; j < UserService.email.length; j++) {
                            if (newLocationSearcheditingEmailArray[i] === UserService.email[j])
                                found = true;
                        }
                        if (!found)
                            newLocationSearcheditingEmailArray.splice(i, 1);
                    }
                    var newLocationSearcheditingEmailString = newLocationSearcheditingEmailArray.join('#');
                    if (newLocationSearcheditingEmailString !== '') {
                        locationSearch[prop] = newLocationSearcheditingEmailString;
                    } else
                        delete locationSearch[prop];
                }
            }
            $location.search(locationSearch);
        }

        function updateEditingDisplayName() {
            $scope.waitingForServerEditDisplayName = false;
            $scope.editDisplayNameAjaxError = false;
            if ($location.search().editingDisplayName) {
                $scope.editDisplayNameText = $scope.displayName;
                $scope.editingDisplayName = true;
            } else
                $scope.editingDisplayName = false;
        }

        function updateAddingEmail() {
            $scope.waitingForServer = false;

            if ($location.search().addingEmail)
                $scope.addingEmail = true;
            else
                $scope.addingEmail = false;
        }

        $scope.principal = UserService.principal;
        $scope.displayName = UserService.displayName;
        $scope.email = UserService.email;

        $scope.addingEmail = false;

        $scope.addNewEmail = function() {
            $location.search().addingEmail = true;
            $location.search($location.search());
        };

        $scope.terminateAddEmail = function() {
            $scope.ajaxError = false;
            delete $location.search().addingEmail;
            $location.search($location.search());
        };

        $scope.addEmail = function(newEmail) {
            $scope.waitingForServer = true;
            var user = new User();
            user.emails = UserService.email.slice(0);
            user.emails.push(newEmail);

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success) {
                        UserService.email.push(newEmail);
                        $scope.terminateAddEmail();
                    } else {
                        //server reports an handeled error
                        $scope.waitingForServer = false;
                        $scope.ajaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServer = false;
                    $scope.ajaxError = true;
                });
        };

        $scope.editDisplayName = function() {
            $location.search().editingDisplayName = true;
            $location.search($location.search());
        };

        $scope.terminateEditDisplayName = function() {
            delete $location.search().editingDisplayName;
            $location.search($location.search());
        };

        $scope.updateDisplayName = function(newDisplayName) {
            $scope.waitingForServerEditDisplayName = true;
            var user = new User();
            user.displayName = newDisplayName;

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success) {
                        $scope.displayName = UserService.displayName = newDisplayName;
                        $scope.terminateEditDisplayName();
                    } else {
                        //server reports an handeled error
                        $scope.waitingForServerEditDisplayName = false;
                        $scope.editDisplayNameAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServerEditDisplayName = false;
                    $scope.editDisplayNameAjaxError = true;
                });
        };

        if (Settings.getAnimationStatus()) {
            $scope.animateAccountDetails = 'animated fadeInDown';
        }

        $scope.$on('$routeChangeStart', function(event, next, current) {
            if (Settings.getAnimationStatus()) {
                $scope.animateAccountDetails = 'animated fadeOutUp';
            }
        });

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            if (Settings.getAnimationStatus()) {
                $scope.animateAccountDetails = 'animated fadeInDown';
            }
            updateAddingEmail();
            updateEditingDisplayName();
            cleanOutdatedSearchPathEmails();
        });

        $scope.$on('$routeUpdate', function(event, route) {
            updateAddingEmail();
            updateEditingDisplayName();
            cleanOutdatedSearchPathEmails();
        });
    }
]);

SCHEMAcat.controller('EmailInstanceCtrl', ['$scope', '$location', 'User', 'UserService',
    function($scope, $location, User, UserService) {

        function updateeditingEmail() {
            $scope.waitingForServerEditEmail = false;
            if ($location.search().editingEmail !== undefined && $location.search().editingEmail.indexOf($scope.emailInstance) !== -1) {
                $scope.editEmailText = $scope.emailInstance;
                $scope.editingEmail = true;
            } else
                $scope.editingEmail = false;
        }

        updateeditingEmail();

        $scope.editEmail = function(emailInstance) {
            var locationSearch = $location.search();
            $scope.removeEmailAjaxError = false;
            if (locationSearch.editingEmail === undefined)
                locationSearch.editingEmail = emailInstance;
            else
                locationSearch.editingEmail = $location.search().editingEmail + '#' + emailInstance;

            $location.search(locationSearch);
        };

        $scope.removeEmail = function(index) {
            $scope.waitingForServerEditEmail = true;

            var user = new User();
            user.emails = UserService.email.slice(0);
            user.emails.splice(index, 1);

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success) {
                        UserService.email.splice(index, 1);
                    } else {
                        //server reports an handeled error
                        $scope.waitingForServerEditEmail = false;
                        $scope.removeEmailAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServerEditEmail = false;
                    $scope.removeEmailAjaxError = true;
                });
        };

        $scope.updateEmail = function(index) {
            $scope.waitingForServerEditEmail = true;

            var user = new User();
            user.emails = UserService.email.slice(0);
            user.emails[index] = $scope.editEmailText;

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success) {
                        var newEmail = $scope.editEmailText;
                        $scope.cancelEditEmail(UserService.email[index]);
                        UserService.email[index] = newEmail;
                    } else {
                        //server reports an handeled error
                        $scope.waitingForServerEditEmail = false;
                        $scope.editEmailAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServerEditEmail = false;
                    $scope.editEmailAjaxError = true;
                });
        };


        $scope.cancelEditEmail = function(emailInstance) {
            var locationSearch = $location.search();
            $scope.editEmailAjaxError = false;

            for (var prop in locationSearch) {
                if (prop === 'editingEmail') {
                    var newLocationSearcheditingEmailArray = locationSearch[prop].split('#');
                    for (var i in newLocationSearcheditingEmailArray) {
                        if (newLocationSearcheditingEmailArray[i] === emailInstance)
                            newLocationSearcheditingEmailArray.splice(i, 1);
                    }
                    var newLocationSearcheditingEmailString = newLocationSearcheditingEmailArray.join('#');
                    if (newLocationSearcheditingEmailString !== '') {
                        locationSearch[prop] = newLocationSearcheditingEmailString;
                    } else
                        delete locationSearch[prop];
                }
            }
            $location.search(locationSearch);
        };

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            updateeditingEmail();
        });

        $scope.$on('$routeUpdate', function(event, route) {
            updateeditingEmail();
        });
    }
]);

//SCHEMAcat controlers for completeUserDetails.html page
SCHEMAcat.controller('CompleteUserDetailsCtrl', ['$scope', '$modal',
    function($scope, $modal) {

        $scope.ajaxError = false;
        $scope.waitingForServer = false;

        var userDetailsModalInstance = $modal.open({
            scope: $scope,
            backdrop: 'static',
            keyboard: false,
            templateUrl: 'partials/fillDetailsModal.html',
            controller: 'CompleteUserDetailsModalCtrl'
        });
    }
]);

SCHEMAcat.controller('CompleteUserDetailsModalCtrl', ['$scope', '$modalInstance', 'User', 'UserService',
    function($scope, $modalInstance, User, UserService) {

        $scope.userDetailsSubmit = function() {
            $scope.waitingForServer = true;
            $scope.ajaxError = false;

            var user = new User();

            user.emails = [];
            user.emails[0] = this.emailInput.text;
            user.displayName = this.displayNameInput.text;

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'] && result['sc:response'].success)
                        document.location.pathname = 'schemacat/site/index.html';
                    else {
                        //server reports an handeled error
                        $scope.waitingForServer = false;
                        $scope.ajaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServer = false;
                    $scope.ajaxError = true;
                });
        };
    }
]);